#include "unary_arithmetic_filter.hpp"
#include "workflow_graph.hpp"
#include "yacc_var.hpp"
#include "file.hpp"

namespace xios
{
  CUnaryArithmeticFilter::CUnaryArithmeticFilter(CGarbageCollector& gc, const std::string& op)
    : CFilter(gc, 1, this)
    , op(operatorExpr.getOpField(op))
  { 
    expression.assign(*yacc_globalInputText_ptr, 0, yacc_globalInputText_ptr->size()-1);
  };

  CDataPacketPtr CUnaryArithmeticFilter::apply(std::vector<CDataPacketPtr> data)
  {
    bool building_graph = this->tag ? data[0]->timestamp >= this->start_graph && data[0]->timestamp <= this->end_graph : false;
    int unique_filter_id;
    
    if(building_graph)
    {
      CWorkflowGraph::allocNodeEdge();
      size_t filterhash = std::hash<StdString>{}(expression+to_string(data[0]->timestamp)+this->field->getId());

      // first round
      if(CWorkflowGraph::mapHashFilterID_ptr->find(filterhash) == CWorkflowGraph::mapHashFilterID_ptr->end())
      {
        this->filterID = InvalidableObject::filterIdGenerator++;
        int edgeID = InvalidableObject::edgeIdGenerator++;

        CWorkflowGraph::addNode(this->filterID, "Arithmetic Filter\\n("+expression+")", 3, 1, 0, data[0]);
        (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].filter_tag = this->tag;

        (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].attributes = this->field->recordXiosAttributes();
        if(this->field->file) (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].attributes += "</br>file attributes : </br>" +this->field->file->recordXiosAttributes();
      
        if(CWorkflowGraph::build_begin)
        {

          CWorkflowGraph::addEdge(edgeID, this->filterID, data[0]);
          (*CWorkflowGraph::mapFilters_ptr_with_info)[this->filterID].expected_entry_nb ++;

          (*CWorkflowGraph::mapFilters_ptr_with_info)[data[0]->src_filterID].filter_filled = 0 ; 
        }
        else CWorkflowGraph::build_begin = true;

        (*CWorkflowGraph::mapHashFilterID_ptr)[filterhash] = this->filterID; 
        unique_filter_id = this->filterID;
      }
      else 
      {
        unique_filter_id = (*CWorkflowGraph::mapHashFilterID_ptr)[filterhash];
        if(data[0]->src_filterID != unique_filter_id)
        {
          int edgeID = InvalidableObject::edgeIdGenerator++;
          CWorkflowGraph::addEdge(edgeID, unique_filter_id, data[0]); 
          (*CWorkflowGraph::mapFilters_ptr_with_info)[data[0]->src_filterID].filter_filled = 0 ; 
          (*CWorkflowGraph::mapFilters_ptr_with_info)[unique_filter_id].expected_entry_nb ++;
        }
      
        
      } 
  
    }

    CDataPacketPtr packet(new CDataPacket);
    packet->date = data[0]->date;
    packet->timestamp = data[0]->timestamp;
    packet->status = data[0]->status;
    packet->field = this->field;

    if (packet->status == CDataPacket::NO_ERROR)
      packet->data.reference(op(data[0]->data));

    if(building_graph) packet->src_filterID = unique_filter_id;

    return packet;
  }
} // namespace xios

